>[danger] 需要引入头文件 `string`

## 一.初始化
1.	**默认初始化** 没有初始化的变量
2.	**复制初始化** 使用等号初始化一个变量
3.	**直接初始化** 不使用等号初始化一个变量
```c++
//默认初始化
string s0;	//默认初始化，s0为空字符串

//复制初始化
string s1 = "Hello World";	//s1的内容为 "Hello World"
string s2 = s1; //s2的内容也为"Hello World"

//直接初始化
string s3("Hello World");	//s3的内容为 "Hello World"
string s4(s1); //s4的内容也为"Hello World"
string s5(5,'a'); //s5的内容为 "aaaaa"
```
## 二.字符串的输入和输出
1.	**输出** 直接用 `cout` 输出即可。
```c++
string s = "20200202";
cout<<s;
```
2.	**输入** 
+	使用 `cin` 输入：输入空白字符表示输入结束。
```c++
string s = "20200202";
cin>>s;
```
+	使用 `getline` 函数输入：输入自定义的结束符表示输入结束。
	**格式** `输入流 getline(输入流,字符串,分隔字符 = '\n')`
```c++
string s;
getline(cin,s,'\n');
```
>[danger] 注意区分 `cin.getline` 与 `getline`。

## 三.字符串的操作

1.	**赋值** 使用 `赋值运算符=` 赋值，类似 `strcpy`
```c++
string s1,s2 = "Hello";
s1 = s2;
```

2.	**比较** 
	+	判断字符串相等 `s1==s2` `s1!=s2`
	+	字符串按字典顺序比较 `s1>s2`  `s1<s2`  `s1>=s2`  `s1<=s2` 

3.	**连接/追加** 末尾插入
	+	使用 `+` 或 `+=` 运算符将 字符串对象 与 另一字符串对象、字符串常量、字符数组、字符指针 、字符 相加。
	+	使用成员函数追加
		+	追加一个字符 `string& string_obj.push_back(char ch)`
		+	追加n个相同的字符 `string& string_obj.append(size_type n, char ch)`
		+	追加 字符串对象、字符串常量、字符数组、字符指针
			+	`string& string_obj.append(const string& s, size_type pos = 0, size_type count = npos)` 
			+	**描述** 在后面追加 `s` 中从 `pos` 开始，总共 `min(count, size() - index)` 个字符。
		> **npos** 是特殊值 -1 ，等于 size_type 类型可表示的最大值。虽然定义使用 -1 ，由于有符号到无符号隐式转换，且 size_type 是无符号整数类型， npos 的值是其所能保有的最大正值。

>[warning]使用运算符和成员函数追加的区别：
>+	**+** 得到一个新的字符串对象，不会修改本身。
>+	**+=** 和 **成员函数** 直接修改自身，不会创建新的字符串对象。

```c++
string s1 = "Foo",s2 = "Bar";
cout<<s1 +' '+ s2<<endl;//Foo Bar
cout<<s1<<endl; //Foo
s2.append(3,'6');
cout<<s2<<endl;//Bar666
```
4.	**删除** `string& string_obj.erase( size_type index = 0, size_type count = npos )` 删除从 `index` 开始，总共 `min(count, size() - index)` 个字符。

5.	**查找** 
	+	`size_type string_obj.find( const string& str, size_type pos = 0 )` 从 pos 开始寻找等于 str 的首个子串。
	+	`size_type string_obj.find( char ch, size_type pos = 0 )` 从 pos 开始寻找等于ch的首个位置。
	+	`size_type string_obj.find( const CharT* s, size_type pos, size_type count )`  寻找等于范围 [s, s+count) 的首个子串。此范围能含空字符。
	+	`size_type string_obj.find( const CharT* s, size_type pos = 0 )` 寻找等于 s 所指向的字符串的首个子串。

## 四.字符串的容量

1. 	**字符串是否为空** `string_obj.empty()`
```c++
string s = "";
cout<<s.empty();	//输出：1
```
2.	**字符串长度** `string_obj.size()` 或 `string_obj.length()` 返回字符的个数。
	+	不含 空字符
	+	它的返回值类型为 `string::size_type` 类型，它是一种 **无符号类型** 。
```c++
string s = "20200202";
cout<<s.size(); //输出8
cout<<( s.size() < -1); //输出1
```
3.	**字符串的最大长度** `string_obj.max_size()` 系统或库实现限制所能保有的最大元素数
	+	它的返回值类型为 `string::size_type` 类型，它相当于一个 **无符号类型** 。
	+	它的大小与 **系统和库实现** 有关，与字符串本身的长度无关。
	+	若操作将导致 `string_obj.size() > string_obj.max_size()` ，则抛出 `std::length_error` 异常 。
## 五.访问字符串中的字符
>[warning] 访问范围 `[0,string_obj.size()]`
>
>+	当 访问第 `string_obj.size()` 个元素时，返回空字符

1.	**下标访问** `string_obj[size_type pos]`
	>[warning] 该方法不做范围检查

	>[success]返回一个引用，因此可以放在等号左边。

2.	`[$]` **at方法访问** `string_obj.at(size_type pos)`
	>[warning] **该方法会做范围检查** 非法访问时抛出 `std::out_of_range` 类型的异常。

	>[success]该方法返回一个引用，因此可以放在等号左边。
	
```c++
string s1 = "2020";
s1.at(3) = '8';//ok
```

